/**
 * file:    ObserverInterface.java
 *
 * classes: interface ObserverInterface
 *
 * purpose: - to provide an interface for an observer so that objects of an implementation of
 *            this interface can be activated by RMI (remote method invocation)
 *            calls
 *
 * date:    05/03/97  last change: 05/03/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */


package observer.generic;

import java.rmi.*;

/**
 * interface ObserverInterface
 *
 * - interface for an observer (observer pattern)
 *
 */
public interface ObserverInterface extends Remote {

    /*
     * returns the unique account of the observer (usually a username)
     */
    String getAccount() throws RemoteException;

    /*
     * returns the current aspect of the observer (for example to indicate different observer-roles)
     */
    int getAspect() throws RemoteException;

    /*
     * method is called when a user has logged in
     */
    void observerLoggedIn(String account) throws RemoteException;

    /*
     * method is called when a user has logged out
     */
    void observerLoggedOut(String account) throws RemoteException;
    
    /*
     * updates the state of the observer
     */
    void update(SubjectInterface si, int aspect) throws RemoteException;
}
