/**
 * file:    SubjectInterface.java
 *
 * classes: interface SubjectInterface
 *
 * purpose: - to provide an interface so that objects of an implementation of
 *            this interface can be activated by RMI (remote method invocation)
 *            calls
 *
 * date:    05/03/97  last change: 05/03/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */


package observer.generic;

import java.rmi.*;
import java.util.*;

/**
 * interface SubjectInterface
 *
 * - interface for a subject (observer pattern)
 *
 */
public interface SubjectInterface extends Remote {

    /*
     * login - method for an observer to login into a subject (considered as server)
     */
    void login(ObserverInterface observer) throws RemoteException, AlreadyLoggedInException;

    /*
     * logout - method for an observer to logout
     */
    void logout(ObserverInterface observer) throws RemoteException, NotLoggedInException;
    
    /*
     * countObservers - count the registered observers
     */
    int countObservers() throws RemoteException;
    
    /*
     * getObserverAccounts - returns an array of strings of accounts of all observers currently logged in 
     */
    Vector getObserverAccounts() throws RemoteException;

    /*
     * notifyChanges - method to notify changes in observers to their subject (normally initiates an observer update)
     */
    void notifyChanges(ObserverInterface observer, int aspect) throws RemoteException;

}
