/**
 * file:    ViewSubjectInterface.java
 *
 * classes: interface ViewSubjectInterface
 *
 * purpose: - as an extension of the SubjectInterface clients that implement this interface
 *            work as Subject for observers (unique identified) and viewers (not identified at all)
 *
 * date:    05/03/97  last change: 05/03/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */


package observer.generic;

import java.rmi.*;

/**
 * interface ViewSubjectInterface
 *
 * - interface for a viewer (observer pattern)
 *
 */
public interface ViewSubjectInterface extends SubjectInterface {

    /*
     * registerViewer - method for an observer to login into a subject (considered as server)
     * Note: Since we don't have a safe login mechanism as in
     *       login (see SubjectInterface) a viewer can register several times.
     *       However this is not recommended, because viewers registered more than once
     *       will receive multiple update commands
     */
    void registerViewer(ObserverInterface observer) throws RemoteException;
}
