/**
 * file:    GomukoClient.java
 *
 * classes: class GomukoClient
 *
 * purpose: obvious
 *
 * date:    06/26/97  last change: 06/26/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */


package observer.gomuko;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

import java.awt.*;
import java.awt.event.*;

import observer.generic.*;
import observer.main.*;

/**
 * class GomukoClient
 *
 */
public final class GomukoClient extends ApplicationClient implements GomukoClientInterface {

    private GomukoFrame gomukoFrame = null;
    private boolean firstGame = true;
    private boolean myTurn = false;
    private boolean finished = false;
    private boolean won = false;
    
    public GomukoClient() throws RemoteException {
    
        super();
    
    }
    
    /*
     * destroy - to destroy the application client
     */
    public void destroy() throws RemoteException {
    
        super.destroy();
        
        if(gomukoFrame != null) {
            gomukoFrame.dispose();
            gomukoFrame = null;
        }

    }
    
    public void yourTurn() throws RemoteException {

        myTurn = true;
        
        if(gomukoFrame != null)
            gomukoFrame.yourTurn(true);
    
    }
    
    public boolean isYourTurn() throws RemoteException {

        return myTurn;
    
    }
        
    public void fieldClicked(int i, int j) throws RemoteException {

        if(myTurn) {
            myTurn = false;

            try {
                ((GomukoServerInterface)server).fieldClicked(i, j);
            }
            catch(Exception e) {
                System.out.println("GomukoClient.initRequestFrame: an exception occurred: " + "Could not determine participants." + 
                                   e.getMessage());
                e.printStackTrace();
                return;
            }
            
            gomukoFrame.yourTurn(false);

        }
    
    }
    
    public void setField(int i, int j, int mode) throws RemoteException {

        gomukoFrame.setField(i, j, mode);
    
    }
    
    public void escapeFrame() throws RemoteException {

        try {
            mainClient.destroyClient(this, true);
        }
        catch(Exception e) {
            System.out.println("GomukoClient.escapeFrame: an exception occurred: " + "Could not determine participants." + 
                               e.getMessage());
            e.printStackTrace();        
        }
    
    }
    
    public synchronized void finishGame(boolean w) throws RemoteException {

        finished = true;
        won = w;
        
        notifyAll();

    }
    
    public synchronized void actionPerformed(ActionEvent actionEvent) {

        Object source = actionEvent.getSource();
        
        super.actionPerformed(actionEvent);
    
    }

    public void run() {

        super.run();
        
        if(isDead || (accepted == NOTACCEPTED))
            return;
            

        try {
            gomukoFrame = new GomukoFrame(this, ((GomukoServerInterface)server).getFieldSize());
        }
        catch(RemoteException e) {
             System.out.println("GomukoClient.run: an exception occurred: " + "Could not determine participants." + 
                                e.getMessage());
             e.printStackTrace();
        }
        gomukoFrame = (GomukoFrame)gomukoFrame.executeFrame(null);
        
        while(true) {
            synchronized(this) {
                
                try {
                    while(!isDead && !finished)
                        wait();
                }
                catch(Exception e) {
                    System.out.println("GomukoClient.run: an exception occurred: " +
	                                   e.getMessage());
                    e.printStackTrace();
                }
                
                if(isDead)
                    return;
        
            }
            
            gomukoFrame.updateScore(won);
                
            negotiateSession();
            if(isDead || (accepted == NOTACCEPTED))
                return;
            
            finished = false;    
            gomukoFrame.resetGame();
            
        }
                
    }
   
    protected void initRequestFrame() {

        Vector buttons = new Vector();
       
        buttons.addElement(new String("Yes"));
        buttons.addElement(new String("No"));
        
        if(firstGame) {
            firstGame = false;
            sessionRequestedFrame = new SimpleQuestionFrame(this, "Gomuko game...", "The following persons would like start a new gomuko game. Do you accept?", true, buttons);

            try {
                ((SimpleQuestionFrame)sessionRequestedFrame).appendItems(server.getObserverAccounts());
            }
            catch(Exception e) {
                 System.out.println("GomukoClient.initRequestFrame: an exception occurred: " + "Could not determine participants." + 
                                    e.getMessage());
        	     e.printStackTrace();
            }
        } else {            
            if(won)
                sessionRequestedFrame = new SimpleQuestionFrame(this, "Gomuko game...", "You have won the game. Do you want to play again?", false, buttons);
            else
                sessionRequestedFrame = new SimpleQuestionFrame(this, "Gomuko game...", "You have lost the game. Do you want to play again?", false, buttons);
        }

        sessionRequestedFrame = (SimpleQuestionFrame)sessionRequestedFrame.executeFrame(null);
    
    }
        
    public void finalize() {

        System.out.println("GomukoCLient destroyed.");
    
    }

}
