/**
 * file:    GomukoFrame.java
 *
 * classes: class GomukoFrame
 *
 * purpose: - a gomuko frame
 *
 * date:    05/16/97  last change: 05/16/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */

package observer.gomuko;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;

import observer.generic.*;
import observer.main.*;

class GomukoCanvas extends Canvas implements AdjustmentListener, MouseListener {

    int tx = 0;
    int ty = 0;

    boolean sizeKnown = false;
    Dimension preferredSize;
    Dimension paperOrigin;
    Dimension paperSize;
    Dimension tiling;
    Dimension realSize;
    Dimension fieldOrigin;
    Dimension fieldSize;
    Dimension field;

    private URL documentBase;
    private GomukoFrame gomukoFrame;
    private Scrollbar horz, vert;

    private Image ringBookImage;
    private Image woodImage;
    private Image circle[], cross[];
    
    private int fieldArray[][];
    private int myScore = 0;
    private int opponentScore = 0;

    
    public GomukoCanvas(URL dB, Dimension fS, GomukoFrame gF, Scrollbar h, Scrollbar v) {
        
        int i;
        
        documentBase = dB;
        gomukoFrame = gF;
        horz = h;
        vert = v;
        
        horz.addAdjustmentListener(this);
        vert.addAdjustmentListener(this);

        preferredSize = new Dimension(696, 754);
                
        try {
            ringBookImage = Toolkit.getDefaultToolkit().getImage(new URL(dB, "ringBook.jpg"));
            woodImage = Toolkit.getDefaultToolkit().getImage(new URL(dB, "wood.jpg"));
            
            circle = new Image[4];
            for(i = 0; i < 4 ; i ++)
                circle[i] = Toolkit.getDefaultToolkit().getImage(new URL(dB, "circle" + (i + 1) + ".jpg"));
                
            cross = new Image[4];
            for(i = 0; i < 4 ; i ++)
                cross[i] = Toolkit.getDefaultToolkit().getImage(new URL(dB, "cross" + (i + 1) + ".jpg"));

        }
        catch(Exception e) {
            // shouldn't happen
        }
                
        setBackground(Color.white);
        
        paperOrigin = new Dimension();
        paperSize = new Dimension();
        tiling = new Dimension();
        realSize = new Dimension();
        fieldOrigin = new Dimension(10, 32);
        fieldSize = new Dimension(fS);
        field = new Dimension(14, 14);
        
        fieldArray = new int[fieldSize.width][fieldSize.height];
        resetBoard();

        positionImages();
        
        addMouseListener(this);
        
    }

    public Dimension getMinimumSize() {
        return new Dimension(10, 10);
    }

    public Dimension getPreferredSize() {
        return preferredSize;
    }

    public boolean imageUpdate(Image theimg, int infoflags,
                               int x, int y, int w, int h) {

        if (((infoflags & (WIDTH | HEIGHT)) != 0) && (!sizeKnown)) {
            positionImages();
        }
        boolean done = ((infoflags & (ERROR | FRAMEBITS | ALLBITS)) != 0);
        // Repaint immediately if we are done, otherwise batch up
        // repaint requests every 100 milliseconds
        repaint(done ? 0 : 100);
        return !done;

    }

    public void drawField(Graphics g, int i, int j) {
    
        switch(fieldArray[i][j] & 3) {
            case GomukoServer.CIRCLE:
                g.drawImage(circle[(fieldArray[i][j] & 12) >> 2], fieldOrigin.width + i * field.width + 1, fieldOrigin.height + j * field.height + 1, getBackground(), this); 
                break;
                    
            case GomukoServer.CROSS:
                g.drawImage(cross[(fieldArray[i][j] & 12) >> 2], fieldOrigin.width + i * field.width + 1, fieldOrigin.height + j * field.height + 1, getBackground(), this); 
                break;
                    
        }
    
    }
    
    public void drawFivePack(Graphics g, int x, int y) {

        int temp = x;

        for(int i = 0; i < 4; i ++) {
            g.drawLine(temp, y, temp, y + 10);
            temp += 5;
        }

        g.drawLine(x - 3, y + 7, x + 18, y + 3);
    
    }

    
    public void paint(Graphics g) {

        update(g);

    }
    
    public void update(Graphics g) {

        int i, j, temp;
        Dimension addTiling = new Dimension();
        
        if(sizeKnown) {
            g.translate(-tx, -ty);

            addTiling.width = (tx + getSize().width) / woodImage.getWidth(this) + 1;
            addTiling.height = (ty + getSize().height) / woodImage.getHeight(this) + 1;
            
            for(i = 0; i < tiling.width; i ++) {
                g.drawImage(woodImage, i * woodImage.getWidth(this), 0, getBackground(), this); 
            }

            for(i = 0; i < tiling.height; i ++) {
                g.drawImage(woodImage, 0, i * woodImage.getHeight(this), getBackground(), this); 
            }
            
            for(i = tiling.width - 1; i < addTiling.width; i ++)
                for(j = 0; j < tiling.height; j ++)
                    g.drawImage(woodImage, i * woodImage.getWidth(this), j * woodImage.getHeight(this), getBackground(), this);

            for(i = 0; i < addTiling.width; i ++)
                for(j = tiling.height - 1; j < addTiling.height; j ++)
                    g.drawImage(woodImage, i * woodImage.getWidth(this), j * woodImage.getHeight(this), getBackground(), this);

            g.drawImage(ringBookImage, woodImage.getWidth(this), woodImage.getHeight(this), getBackground(), this);
        
            g.clearRect(paperOrigin.width, paperOrigin.height, paperSize.width, paperSize.height);

            // drawing the grid
            g.setColor(new Color(200, 200, 255));
            temp = 0;
            j = field.height * fieldSize.height;
            for(i = 0; i <= fieldSize.width; i ++) {
                g.drawLine(fieldOrigin.width + temp, fieldOrigin.height, fieldOrigin.width + temp, fieldOrigin.height + j);
                temp += field.width;
            }

            temp = 0;
            i = field.width * fieldSize.width;
            for(j = 0; j <= fieldSize.height; j ++) {
                g.drawLine(fieldOrigin.width, fieldOrigin.height + temp, fieldOrigin.width + i, fieldOrigin.height + temp);
                temp += field.height;
            }
                        
            // drawing circles and crosses
            for(i = 0; i < fieldSize.width; i ++)
                for(j = 0; j < fieldSize.height; j ++)
                    drawField(g, i, j);

            // drawing the score
            g.setColor(Color.black);


            // at first mine
            temp = fieldOrigin.width;
            j = myScore / 5;
            for(i = 0; i < j; i ++) {
                drawFivePack(g, temp, fieldOrigin.height - 15);
                temp += 20;
            }
            
            j = myScore % 5;
            
            for(i = 0; i < j; i ++) {
                g.drawLine(temp, fieldOrigin.height - 15, temp, fieldOrigin.height - 5);
                temp += 5;        
            }
            
            // at now the opponents score
            temp = fieldOrigin.width + field.width * fieldSize.width - 20;
            j = opponentScore / 5;
            for(i = 0; i < j; i ++) {
                drawFivePack(g, temp, fieldOrigin.height - 15);
                temp -= 20;
            }
            
            j = opponentScore % 5;
            temp += 15;
            
            for(i = 0; i < j; i ++) {
                g.drawLine(temp, fieldOrigin.height - 15, temp, fieldOrigin.height - 5);
                temp -= 5;        
            }


        }
    }
        
    public void adjustmentValueChanged(AdjustmentEvent adjustmentEvent) {

        Object source = adjustmentEvent.getAdjustable();
        
        if (source == horz) {
            tx = adjustmentEvent.getValue();
            repaint();
        }
        
        if(source == vert) {
            ty = adjustmentEvent.getValue();
            repaint();
        }
        
    }    


    public void mouseClicked(MouseEvent mouseEvent) {
    }
    
    public void mouseEntered(MouseEvent mouseEvent) {
    }
    
    public void mouseExited(MouseEvent mouseEvent) {
    }
    
    public void mousePressed(MouseEvent mouseEvent) {
 
        if((mouseEvent.getX() + tx >= fieldOrigin.width) && (mouseEvent.getX() + tx <= fieldOrigin.width + field.width * fieldSize.width) &&
           (mouseEvent.getY() + ty >= fieldOrigin.height) && (mouseEvent.getY() + ty <= fieldOrigin.height + field.height * fieldSize.height))
            gomukoFrame.fieldClicked((mouseEvent.getX() + tx - fieldOrigin.width) / field.width,
                                     (mouseEvent.getY() + ty - fieldOrigin.height) / field.height);

    }
    
    public void mouseReleased(MouseEvent mouseEvent) {
    }
    
    public void resetBoard () {
    
        for(int i = 0; i < fieldArray.length; i ++)
            for(int j = 0;  j < fieldArray[i].length; j ++)
                fieldArray[i][j] = GomukoServer.NOTOCCUPIED;
    
    }

    public void positionImages() {
        
        if(ringBookImage.getWidth(this) < 0 || ringBookImage.getHeight(this) < 0 || woodImage.getWidth(this) < 0 || woodImage.getHeight(this) < 0)
            return;

        paperOrigin.width = woodImage.getWidth(this) + ringBookImage.getWidth(this);
        paperOrigin.height = woodImage.getHeight(this);
        paperSize.width = (int)((double)ringBookImage.getHeight(this) * 0.7);
        paperSize.height = ringBookImage.getHeight(this);
        
        tiling.width = (ringBookImage.getWidth(this) + paperSize.width) / woodImage.getWidth(this) + 2;
        tiling.height = paperSize.height / woodImage.getHeight(this) + 2;
        
        realSize.width = 2 * woodImage.getWidth(this) + ringBookImage.getWidth(this) + paperSize.width;
        realSize.height = 2 * paperOrigin.height + paperSize.height;

        fieldOrigin.width += paperOrigin.width;
        fieldOrigin.height += paperOrigin.height;
        
        sizeKnown = true;
        
        updateScrollbars();
    }
    
    public void updateScrollbars() {
    
        Dimension canvasSize = new Dimension();

        canvasSize = getSize();
        
        if (canvasSize.width <= 0)
            return;

        //Shift everything to the right if we're displaying empty space
        //on the right side.
        if ((tx + canvasSize.width) > realSize.width) {
            int newtx = realSize.width - canvasSize.width;
            if (newtx < 0) {
                newtx = 0;
            }
            tx = newtx;
        }

        horz.setValues(//draw the part of the image that starts at this x:
                       tx, 
                       //amount to scroll for a "page":
                       canvasSize.width, 
                       //minimum image x to specify:
                       0,
                       //maximum image x to specify:
                       realSize.width);
        //"visible" arg to setValues() has no effect after scrollbar is visible.
        horz.setBlockIncrement(canvasSize.width);

        if (canvasSize.height <= 0)
            return;

        //Shift everything downward if we're displaying empty space
        //on the bottom.
        if ((ty + canvasSize.height) > realSize.height) {
            int newty = realSize.height - canvasSize.height;
            if (newty < 0) {
                newty = 0;
            }
            ty = newty;
        }

        vert.setValues(//initially draw part of image starting at this y:
                       ty, 
                       //visible arg--amount to scroll for a "page":
                       canvasSize.height, 
                       //minimum image y to specify:
                       0,
                       //maximum image y to specify:
                       realSize.height);

        //"visible" arg to setValues() has no effect after scrollbar is visible.
        vert.setBlockIncrement(canvasSize.height);
    
    }
    
    public void setField(int i, int j, int mode) {

        Graphics g;
        
        fieldArray[i][j] = mode;
        g = getGraphics();
        
        if(g != null) {
            
            g.translate(-tx, -ty);

            drawField(g, i, j);            
        }
    
    }
    
    public void updateScore(boolean won) {

        if(won)
            myScore ++;
        else
            opponentScore ++;
            
    }
        
}

public class GomukoFrame extends CenteredFrame implements ActionListener {
    
    protected FramedPanel mainPanel;

    protected Scrollbar vert;
    protected Scrollbar horz;
    protected GomukoCanvas gomukoCanvas;
    
    protected FramedPanel buttonPanel;
    protected Label messageLabel = null;
    protected Button giveUpButton;
    
    protected GomukoClientInterface gomukoClient;
    
    protected boolean myTurn = false;

    public GomukoFrame(GomukoClientInterface gC, Dimension fS) {

        super("Gomuko");

        gomukoClient = gC;

        mainPanel = new FramedPanel(FramedPanel.NOFRAME);

        horz = new Scrollbar(Scrollbar.HORIZONTAL);
        vert = new Scrollbar(Scrollbar.VERTICAL);

        try {
            gomukoCanvas = new GomukoCanvas(gC.getDocumentBase(), fS, this, horz, vert);
        }
        catch(Exception e) {
            // can't happen
        }

        buttonPanel = new FramedPanel(FramedPanel.FRAMEIN);
        messageLabel = new Label();
        giveUpButton = new Button("Give Up!");

        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Insets insets = new Insets(0, 0, 0, 0);

        mainPanel.setLayout(gbl);

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbl.setConstraints(gomukoCanvas, gbc);

        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.weightx = 0;
        gbc.weighty = 1;
        gbl.setConstraints(vert, gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        gbc.weighty = 0;
        gbl.setConstraints(horz, gbc);

        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        gbc.weighty = 0;
        gbl.setConstraints(messageLabel, gbc);

        buttonPanel.setLayout(new GridLayout(1, 0));
        buttonPanel.add(giveUpButton);

        gbc.gridx = 0;
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.gridwidth = 0;
        gbc.weightx = 1;
        gbc.weighty = 0;
        insets.top = 10;
        gbc.insets = insets;
        gbl.setConstraints(buttonPanel, gbc);

        mainPanel.add(gomukoCanvas);
        mainPanel.add(vert);
        mainPanel.add(horz);
        mainPanel.add(messageLabel);
        mainPanel.add(buttonPanel);

        addWindowListener(this);
        giveUpButton.addActionListener(this);

        setLayout(new GridLayout(0, 1));
        add(mainPanel);
        
        try {
            yourTurn(gomukoClient.isYourTurn());
        }
        catch(Exception e) {
            // can't happen
        }

        pack();

    }

    public void paint(Graphics g) {

        gomukoCanvas.updateScrollbars();
    
    }
    
    public void enterFrame() {


    }
    
    public void escapeFrame() {

        dispose();

        try {
            gomukoClient.escapeFrame();
        }
        catch(Exception e) {
            // can't happen
        }
        
    }
    
    public void actionPerformed(ActionEvent e) {

        Object source = e.getSource();

        if(source == giveUpButton)
            escapeFrame();

    }
    
    public void fieldClicked(int i, int j) {

        System.out.println("Feld angeklickt: " + i + ", " + j + ".");
        
        try {
            gomukoClient.fieldClicked(i, j);
        }
        catch(Exception e) {
            // can't happen
        }
        
    }
    
    public void setField(int i, int j, int mode) {

        gomukoCanvas.setField(i, j, mode);
    
    }
    
    public void resetGame() {

        gomukoCanvas.resetBoard();
        gomukoCanvas.repaint();
        
    }
    
    public void updateScore(boolean won) {

        gomukoCanvas.updateScore(won);
        
    }
    
    public void yourTurn(boolean mT) {

        myTurn = mT;
        
        if(messageLabel != null) {
        
            if(myTurn)
                messageLabel.setText("Your turn.");
            else
                messageLabel.setText("Opponents turn.");
        
        }
    
    }
    
}
