/**
 * file:    GomukoServer.java
 *
 * classes: class GomukoServer
 *
 * purpose: obvious
 *
 * date:    06/26/97  last change: 06/26/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */


package observer.gomuko;

import java.awt.*;

import java.rmi.*;
import java.util.*;

import observer.generic.*;
import observer.main.*;

/**
 * class GomukoServer
 *
 */
public final class GomukoServer extends ApplicationServer implements GomukoServerInterface {
    
    public static final int FIELDSET    = 0x0201;
    
    public static final int NOTOCCUPIED = 0x0001;
    public static final int CIRCLE      = 0x0002;
    public static final int CROSS       = 0x0003;
    
    protected static final int VERTICAL   = 0x0001;
    protected static final int DIADOWN    = 0x0002;
    protected static final int HORIZONTAL = 0x0003;
    protected static final int DIAUP      = 0x0004;

    private Dimension fieldSize;
    private int fieldArray[][];
    
    private Vector players;
    private int turn;
    private boolean playerHasSet;
    private boolean finished = false;
    private boolean won = false;
    
    public GomukoServer() throws RemoteException {

        super();
        
        fieldSize = new Dimension(28, 40);
        fieldArray = new int[fieldSize.width][fieldSize.height];
        
        players = new Vector();
        
        resetGame();
    }
        
    protected void updateObserver(ObserverInterface observer, int aspect, Object param) throws RemoteException {

        if(observer instanceof GomukoClientInterface) {
            
            GomukoClientInterface gomukoClient = (GomukoClientInterface)observer;
            
            switch(aspect) {
                
                case FIELDSET:
                    Point p = (Point)param;
                    
                    gomukoClient.setField(p.x, p.y, fieldArray[p.x][p.y]);
                
                default:
                    super.updateObserver(observer, aspect, param);
            
            }
            
        } else
            super.updateObserver(observer, aspect, param);            
            
    }
    
    protected ObserverStorage newObserverStorage(String a, ObserverInterface o) {

        return new GomukoClientStorage(a, o);
    
    }
        
    /*
     * getInformation() - returns specific information about this subject
     */
    public String getInformation() throws RemoteException {
        
        String s = new String("This is a gomuko server. ");
        
        if(players.size() == 2)
            return(new String(s + "Currently there are " + (String)players.elementAt(0) + " and " + (String)players.elementAt(1) + " playing this game."));
        else
            return(new String(s + "Currently there are no players playing this game."));
        
    }
    
    public Dimension getFieldSize() throws RemoteException {
    
        return fieldSize;
    
    }
    
    public void fieldClicked(int i, int j) throws RemoteException {

        Point p = new Point(i, j);
        int counter = 0;
        int random = (int)(Math.random() * (double)3.0);
        
        fieldArray[i][j] = (random << 2) + turn;
        
        broadcast(FIELDSET, p);
        
        counter = Math.max(counter, checkBoard(p, VERTICAL));
        counter = Math.max(counter, checkBoard(p, DIADOWN));
        counter = Math.max(counter, checkBoard(p, HORIZONTAL));
        counter = Math.max(counter, checkBoard(p, DIAUP));

        if(counter == 5) {
            finished = true;
            won = true;
        }
            
        if(counter > 5) {
            finished = true;
            won = false;
        }
        
        synchronized(this) {
            playerHasSet = true;
            notifyAll();
        }
        
    }
    
    public void run() {
        
        Enumeration enum;
        GomukoClientInterface gC;

        super.run();
        
        if(isDead)
            return;
            
        observersCritical.enter();
        
        enum = observers.elements();
        
        while(enum.hasMoreElements()) {
            gC = (GomukoClientInterface)((GomukoClientStorage)enum.nextElement()).getObserver();

            try {
                players.addElement(gC.getAccount());
            }
            catch(Exception e) {
                System.out.println("GomukoServer.run: an exception occurred: " +
	    	                        e.getMessage());
    	        e.printStackTrace();
            }
        }

        observersCritical.leave();
        
        finished = false;
        turn = CROSS;
        
        while(true) {
        
            playerHasSet = false;

            observersCritical.enter();
        
            gC = (GomukoClientInterface)((GomukoClientStorage)observers.get(players.elementAt((turn == CROSS)? 0 : 1))).getObserver();
            
            try {
                gC.yourTurn();
            }
            catch(Exception e) {
                    System.out.println("GomukoServer.run: an exception occurred: " +
	    	                            e.getMessage());
        	        e.printStackTrace();
            }
        
            observersCritical.leave();
        
            synchronized(this) {
            
                try {
                    while(!isDead && !playerHasSet)
                        wait();
                }
                catch(Exception e) {
                    System.out.println("GomukoServer.run: an exception occurred: " +
	    	                            e.getMessage());
        	        e.printStackTrace();
                }
                
                if(isDead)
                    return;
        
            }
        
            if(finished) {
    
                // if the player on turn has six orr more in a row we have to flip the order
                // so that the loser begins the next game
                if(!won)
                    turn = (turn == CROSS) ? CIRCLE : CROSS;
                
                negotiateSession();
                if(isDead)
                    return;
                                
                finished = false;    
                resetGame();
            }
            
            turn = (turn == CROSS) ? CIRCLE : CROSS;
        }

    }
    
    public void resetGame() {
    
        for(int i = 0; i < fieldArray.length; i ++)
            for(int j = 0;  j < fieldArray[i].length; j ++)
                fieldArray[i][j] = NOTOCCUPIED;
    
    }

    protected Point moveToField(Point start, int direction, boolean left) {
        
        Point dif = new Point(0, 0);
        
        switch(direction) {

            case VERTICAL:
                dif.y = 1;
                break;
                
            case DIADOWN:
                dif.x = 1;
                dif.y = 1;
                break;            
                
            case HORIZONTAL:
                dif.x = 1;
                break;            
                
            case DIAUP:
                dif.x = 1;
                dif.y = -1;
                break;
                
        }

        dif.x *= (left) ? -1 : 1;
        dif.y *= (left) ? -1 : 1;
        
        return new Point(start.x + dif.x, start.y + dif.y);
    }
    
    protected int checkBoard(Point start, int direction) {

        Point temp = new Point(start);
        int player = fieldArray[start.x][start.y] & 3;
        boolean first = true;
        int counter = 1;
        
        if(player == NOTOCCUPIED)
            return 0;
                
        while(true) {
            while(true) {
                temp = moveToField(temp, direction, first);
            
                if((temp.x >= 0) && (temp.x < fieldSize.width) && (temp.y >= 0) && (temp.y < fieldSize.height)) {
                    if((fieldArray[temp.x][temp.y] & 3) == player)
                        counter ++;
                    else
                        break;
                } else
                    break;
            }
            
            if(!first)
                break;
            else {
                first = false;
                temp = new Point(start);
            }
        }
        
        return counter;
            
    }
    
    protected void informClient(ApplicationClientInterface aC) throws RemoteException {

        GomukoClientStorage wS, lS;
        GomukoClientInterface winner, loser;

        if(finished) {
            observersCritical.enter();
        
            wS = (GomukoClientStorage)observers.get(players.elementAt((turn == CROSS)? 0 : 1));
            lS = (GomukoClientStorage)observers.get(players.elementAt((turn == CROSS)? 1 : 0));
            
            wS.won();
        
            winner = (GomukoClientInterface)wS.getObserver();
            loser  = (GomukoClientInterface)lS.getObserver();
            
            try {
                winner.finishGame(true);
                loser.finishGame(false);
            }
            catch(Exception e) {
                    System.out.println("GomukoServer.informClient: an exception occurred: " +
	    	                            e.getMessage());
        	        e.printStackTrace();
            }
        
            observersCritical.leave();
            
        } else
            super.informClient(aC);
    
    }
    
}
