/**
 * file:    ApplicationClientInterface.java
 *
 * classes: interface ApplicationClientInterface
 *
 * purpose: - implementations of this interface are application specific clients
 *
 * date:    05/09/97  last change: 05/03/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */


package observer.main;

import java.rmi.*;
import java.util.*;
import java.net.*;

import observer.generic.*;

/**
 * interface ApplicationClientInterface
 *
 */
public interface ApplicationClientInterface extends ObserverInterface {

    /*
     * init - method as replacement for a constructor. This is necessary because we need a no-arg constructor
     */
    void init(MainClientInterface mC, ApplicationServerInterface aS, URL dB) throws RemoteException;
    
    /*
     * start - to start the application client
     */
    void start() throws RemoteException;
    
    /*
     * destroy - to destroy the application client
     */
    void destroy() throws RemoteException;
    
    void clientsHaveAccepted() throws RemoteException;
    
    /*
     * isDead - returns true if the client has been cut off the server
     */
    boolean isDead() throws RemoteException;
    
    /*
     * getMainClient() - returns the main client
     */
    MainClientInterface getMainClient() throws RemoteException;

    /*
     * getServer - returns the corresponding server to this client
     */
    ApplicationServerInterface getServer() throws RemoteException;
    
    /*
     * message - method to recieve a textual message
     */
    void textMessage(String message) throws RemoteException;

    /*
     * getDocumentBase - returns the document base of the applet 
     */
    URL getDocumentBase() throws RemoteException;

}
