/**
 * file:    CenteredFrame.java
 *
 * classes: class CenteredFrame
 *
 * purpose: - a frame that is automatically positioned in the center of the screen
 *
 * date:    05/16/97  last change: 05/16/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */

package observer.main;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class CenteredFrame extends Frame implements KeyListener, WindowListener {
    
    /*
     * frameExists - attribute to simulate modal dialog boxes
     */
    private Modality modality = null;
    
    public final boolean canExecuteFrame() {

        if(modality != null) {
                return modality.canExecuteFrame();
        } else
            return true;
            
    }
    
    public CenteredFrame(String title) {

        super(title);
    
    }
    
    public synchronized void dispose() {

        if(modality != null)
            modality.leave();
        
        super.dispose();

    }
    
    public void pack() {
    
        Dimension screenSize, componentSize;
        
        super.pack();
        
        screenSize = getToolkit().getScreenSize();
        componentSize = getSize();
        
        setBounds(screenSize.width / 2 - componentSize.width / 2,
                  screenSize.height / 2 - componentSize.height / 2,
                  componentSize.width, componentSize.height);
    
    }
        
    public synchronized CenteredFrame executeFrame(Modality m) {

        modality = m;
        
        if(canExecuteFrame()) {
            show();
            if(modality != null)
                modality.enter();
            return this;
        } else
        {
            super.dispose();
            return null;
        }
    }
    
    public void enterFrame() {
    }
    
    public void escapeFrame() {
    }
    
    public void keyPressed(KeyEvent e) {

        switch(e.getKeyCode()) {
        
            case KeyEvent.VK_ENTER:
                enterFrame();
                break;
                
            case KeyEvent.VK_ESCAPE:
                escapeFrame();
                break;
                
        }

    }
    
    public void keyReleased(KeyEvent e) {
    }

    public void keyTyped(KeyEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }
    
    public void windowClosed(WindowEvent e) {
    }
    
    public void windowClosing(WindowEvent e) {
        
        escapeFrame();
        
    }

    public void windowDeactivated(WindowEvent e) {
    }
    
    public void windowDeiconified(WindowEvent e) {
    }
    
    public void windowIconified(WindowEvent e) {
    }

    public void windowOpened(WindowEvent e) {
    }
    
}
