/**
 * file:    InputFrame.java
 *
 * classes: class InputFrame
 *
 * purpose: - an input frame
 *
 * date:    05/16/97  last change: 05/16/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */

package observer.main;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class InputFrame extends CenteredFrame implements ActionListener, TextListener {
    
    protected boolean valid = false;
    
    protected FramedPanel mainPanel;
    protected Label inputLabel;
    protected TextField inputTextField;
    protected FramedPanel buttonPanel;
    protected Button okButton, cancelButton;
    protected String input = null;
    protected ActionListener actionListener;
    
    public InputFrame(ActionListener l, String title, String iL, String i) {
        
        super(title);
        
        actionListener = l;

        mainPanel = new FramedPanel(FramedPanel.NOFRAME);
        inputLabel = new Label(iL);
        inputTextField = new TextField(i, 40);
        buttonPanel = new FramedPanel(FramedPanel.FRAMEIN);
        okButton = new Button("Ok");
        cancelButton = new Button("Cancel");
        
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Insets insets = new Insets(0, 0, 0, 0);

        mainPanel.setLayout(gbl);

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 0;
        gbl.setConstraints(inputLabel, gbc);    
        
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbl.setConstraints(inputTextField, gbc);
        
        buttonPanel.setLayout(new GridLayout(1, 0));
        buttonPanel.add(okButton);
        buttonPanel.add(cancelButton);
        
        gbc.gridx = 0;
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.gridwidth = 0;
        gbc.weightx = 1;
        gbc.weighty = 0;
        insets.top = 10;
        gbc.insets = insets;
        gbl.setConstraints(buttonPanel, gbc);
        
        mainPanel.add(inputLabel);
        mainPanel.add(inputTextField);
        mainPanel.add(buttonPanel);
    
        addWindowListener(this);
        inputTextField.addKeyListener(this);
        inputTextField.addTextListener(this);
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);
        
        setLayout(new GridLayout(0, 1));
        add(mainPanel);
        
        pack();
        
        updateComponents();
    
    }

    public void enterFrame() {

        if(valid) {
            input = inputTextField.getText();
        
            actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, ""));
        
            dispose();
        }

    }
    
    public void escapeFrame() {

        actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, ""));        
        
        dispose();
        
    }
    
    public void actionPerformed(ActionEvent e) {

        Object source = e.getSource();
        
        if(source == okButton) {
            enterFrame();
            return;
        }

        if(source == cancelButton) {
            escapeFrame();
            return;
        }

    }
    
    public void textValueChanged(TextEvent e) {
        
        Object source = e.getSource();
        
        if(source == inputTextField) {
            valid = (inputTextField.getText().length() != 0);
            updateComponents();
            return;
        }
    }

    
    public String getInput() {
    
        return input;
    
    }
    
    public void updateComponents() {

        okButton.setEnabled(valid);
        
    }    

}

