/**
 * file:    MainApplet.java
 *
 * classes: class MainApplet
 *
 * purpose: - to provide a login screen and to launch other servers
 *
 * date:    05/03/97  last change: 05/03/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */

package observer.main;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.util.*;

import observer.generic.*;

public class MainApplet extends Applet implements MainClientInterface, ActionListener, ItemListener {

    private MainServerInterface mainServer;
    private Vector clients;
    private Critical clientsCritical;

    private String account = null;
    
    private Modality modality = null;
    private InputFrame loginFrame = null;
    private SimpleQuestionFrame logoutFrame = null;
    private NewSessionFrame newSessionFrame = null;
    
    private boolean loggedIn = false;

    private FramedPanel mainPanel, buttonPanel;
    private Label userLabel, sessionLabel, infoLabel;
    private List userList, sessionList;
    private TextArea sessionInfo;
    private Button loginButton, logoutButton, newSessionButton;


    public void init() {

   	    try {
   	        // export the applet as a remote object
	        System.out.println("MainApplet.init: exporting remote object");
	        UnicastRemoteObject.exportObject(this);

	        // lookup server
	        URL base = getDocumentBase();
	        String serverName = "//" + base.getHost() + ":" +
		    getParameter("registryPort") + "/observer.main.MainServer";

	        System.out.println("MainApplet.init: looking up server");
	        mainServer = (MainServerInterface)Naming.lookup(serverName);

	        mainServer.registerViewer(this);
        }
        catch(Exception e) {
            System.out.println("MainServer.init: an exception occurred: " + e.getMessage());
            e.printStackTrace();
            return;
        }

        clients = new Vector();
        clientsCritical = new Critical(1);
        
        modality = new Modality();
        
        userLabel = new Label("Users");
        userList = new List();
        userList.setMultipleMode(true);
        sessionLabel = new Label("Sessions");
        sessionList = new List();
        infoLabel = new Label("Session information");
        sessionInfo = new TextArea("", 2, 40, TextArea.SCROLLBARS_NONE);
        sessionInfo.setEditable(false);
        buttonPanel = new FramedPanel(FramedPanel.FRAMEIN);
        loginButton = new Button("Login");
        logoutButton = new Button("Logout");
        newSessionButton = new Button("New Session");
        mainPanel = new FramedPanel(FramedPanel.FRAMEIN);

        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Insets insets = new Insets(0, 0, 0, 0);

        mainPanel.setLayout(gbl);

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 0;
        gbl.setConstraints(userLabel, gbc);

        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridheight = 3;
        gbc.weighty = 1;
        gbl.setConstraints(userList, gbc);

        gbc.gridx = GridBagConstraints.RELATIVE;
        gbc.gridy = 0;
        gbc.gridheight = 1;
        gbc.weighty = 0;
        gbl.setConstraints(sessionLabel, gbc);

        gbc.gridx = 1;
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.weighty = 1;
        gbl.setConstraints(sessionList, gbc);

        gbc.weighty = 0;
        gbl.setConstraints(infoLabel, gbc);

        gbc.weighty = 1;
        gbl.setConstraints(sessionInfo, gbc);

        buttonPanel.setLayout(new GridLayout(1, 0));
        buttonPanel.add(loginButton);
        buttonPanel.add(logoutButton);
        buttonPanel.add(newSessionButton);

        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.gridwidth = 2;
        gbc.weightx = 1;
        gbc.weighty = 0;
        insets.top = 10;
        gbc.insets = insets;
        gbl.setConstraints(buttonPanel, gbc);

        mainPanel.add(userLabel);
        mainPanel.add(userList);
        mainPanel.add(sessionLabel);
        mainPanel.add(sessionList);
        mainPanel.add(infoLabel);
        mainPanel.add(sessionInfo);
        mainPanel.add(buttonPanel);

        setLayout(new GridLayout(0, 1));
        add(mainPanel);

        loginButton.addActionListener(this);
        logoutButton.addActionListener(this);
        newSessionButton.addActionListener(this);
        sessionList.addItemListener(this);
    
    }

    public void start() {

        updateComponents();
        updateLists();

    }

    public void destroy() {

        logout();

    }
    
    public String getAccount() throws RemoteException {

        return account;

    }

    public int getAspect() throws RemoteException {

        return 0;

    }

    public void observerLoggedIn(String account) throws RemoteException {

        userList.add(account);
    
    }
    
    public void observerLoggedOut(String account) throws RemoteException {
        
        userList.remove(account);
       
    }
    
    public void update(SubjectInterface subject, int aspect) throws RemoteException {
    }
    
    public ApplicationClientInterface startClient(ApplicationServerInterface aS, String clientClassName) throws RemoteException {

        Class clientClass;
        ApplicationClientInterface client;
        
        clientsCritical.enter();
            
        try {
            clientClass = Class.forName(clientClassName);
            clients.addElement(client = (ApplicationClientInterface)clientClass.newInstance());
        }
        catch(Exception e) {
  	        System.out.println("MainApplet.startClient: an exception occurred: " +
                e.getMessage());
    	    e.printStackTrace();
            clientsCritical.leave();
    	    return null;
        }
        
        clientsCritical.leave();
        
        client.init(this, aS, getDocumentBase());
        
        try {
            aS.login(client);
        }
        catch(AlreadyLoggedInException e) {
  	        System.out.println("MainApplet.startClient: an exception occurred: " +
                e.getMessage());
    	    e.printStackTrace();
    	    return null;
        }
    
        System.out.println("Client of type " + clientClassName + " successfully started.");        
        
        return client;

    }
    
    public void destroyClient(ApplicationClientInterface aC, boolean logout) throws RemoteException {

        clientsCritical.enter();
        
        try {
            System.out.println("Destroying " +  aC.getAccount());
            
            if(logout)
            System.out.println("fromMainClient");
            else
            System.out.println("fromServer");
            
            if(logout)
                aC.getServer().logout(aC);
                
        }
        catch(Exception e) {
            System.out.println("MainApplet.destroyClient: an exception occurred: " +
                               e.getMessage());
    	    e.printStackTrace();
            clientsCritical.leave();    	    
    	    return;
        }
        
        aC.destroy();

        clients.removeElement(aC);
        
        clientsCritical.leave();
    
    }
        
    public void sessionCreated(String sessionName) throws RemoteException {

        sessionList.add(sessionName);
    
    }
    
    public void sessionDestroyed(String sessionName) throws RemoteException {
        
        sessionList.remove(sessionName);
        
        // workaround for a bug in JDK 1.1.1
        if(sessionList.getSelectedIndex() == -1)
            sessionInfo.setText("");
       
    }

    
    public void actionPerformed(ActionEvent actionEvent) {

        Object source = actionEvent.getSource();
        
        if((loginFrame == null) && (source == loginButton) && (!loggedIn)) {
            loginFrame = new InputFrame(this, "Login...", "Please identify yourself", "");
            loginFrame = (InputFrame)loginFrame.executeFrame(modality);
            return;
        }

        if((logoutFrame == null) && (source == logoutButton) && (loggedIn)) {
            Vector labels = new Vector(2);
            
            labels.addElement(new String("Ok"));
            labels.addElement(new String("Cancel"));
            
            logoutFrame = new SimpleQuestionFrame(this, "Logout...", "Are you sure?", false, labels);
            logoutFrame = (SimpleQuestionFrame)logoutFrame.executeFrame(modality);
            return;
        }
        
        if(source == newSessionButton) {
            
            Vector v;
            
            try {
                v = mainServer.getInstalledSystems();
            }
            catch(Exception e) {
                System.out.println("MainApplet.actionPerformed: an exception occurred: " +
	    	                        e.getMessage());
            	e.printStackTrace();
            	return;
            }

            newSessionFrame = new NewSessionFrame(this, v);
            newSessionFrame = (NewSessionFrame)newSessionFrame.executeFrame(modality);
        }

        if((loginFrame != null) && (source == loginFrame)) {       
            
            String privateAccount;
            
            if(loginFrame.getInput() != null) 
                privateAccount = new String(loginFrame.getInput());
            else
                privateAccount = null;
            
            loginFrame = null;
 
            if(privateAccount != null) {
                try {
                    mainServer.logout(this);
                    account = privateAccount;
                    mainServer.login(this);
                }
                catch(AlreadyLoggedInException e) {
                    MessageBox messageBox = new MessageBox("Error!", "Another user is logged in using the account \"" + account + "\".");
                    messageBox = (MessageBox)messageBox.executeFrame(null);
                    account = null;
                    
                    try {
                        mainServer.registerViewer(this);
                    }
                    catch(Exception e1) {
                        System.out.println("MainApplet.actionPerformed: an exception occurred: " +
	        	                            e1.getMessage());
        	            e1.printStackTrace();
        	            return;
                    }
                    
                    return;
                }
                catch(Exception e) {
                    System.out.println("MainApplet.actionPerformed: an exception occurred: " +
	    	                            e.getMessage());
        	        e.printStackTrace();
        	        return;
                }
             
                loggedIn = true;
                updateComponents();

            }
            return;
            
        }
        
        if((logoutFrame != null) && (source == logoutFrame)) {

            int button = logoutFrame.getSelectedButton();
            
            logoutFrame = null;
            
            if((loggedIn) && (button == 0)) {
                
                logout();
                
                try {
                    account = null;
                    mainServer.registerViewer(this);
                }
                catch(Exception e) {
                    System.out.println("MainApplet.actionPerformed: an exception occurred: " +
	    	                            e.getMessage());
            	    e.printStackTrace();
            	    return;
            	}
            	
            	loggedIn = false;
                updateComponents();
                updateLists();
            }
            return;             
        }
        
        if((newSessionFrame != null) && (source == newSessionFrame)) {
            
            Vector v;
            
            v = new Vector();
            String[] selectedUserAccounts = userList.getSelectedItems();

            try {
                v.addElement(getAccount());
            }
            catch(Exception e) {
                System.out.println("MainApplet.actionPerformed: an exception occurred: " +
	    	                        e.getMessage());
                e.printStackTrace();
                return;
            }
            
            for(int i = 0; i < selectedUserAccounts.length; i ++)
                if(v.indexOf(selectedUserAccounts[i]) == -1)
                    v.addElement(selectedUserAccounts[i]);

            try {
                if(newSessionFrame.getSessionName() != null)
                    mainServer.startServer(newSessionFrame.getSystemName(), newSessionFrame.getSessionName(), v);
                
                newSessionFrame = null;
            }
            catch(NotLoggedInException e) {
                    MessageBox messageBox = new MessageBox("Error!", "One or more users you want to connect with are no longer logged in!");
                    messageBox = (MessageBox)messageBox.executeFrame(null);            
            }
            catch(AlreadyRunningException e) {
                    MessageBox messageBox = new MessageBox("Error!", "Session is already running");
                    messageBox = (MessageBox)messageBox.executeFrame(null);            
            }
            catch(UserMismatchException e) {
                    MessageBox messageBox = new MessageBox("Error!",
                                                           "The session you wanted to start requires from " + e.getMin() + " up to " + e.getMax() + " users.");
                    messageBox = (MessageBox)messageBox.executeFrame(null);            
            }
            
            catch(Exception e) {
                System.out.println("MainApplet.actionPerformed: an exception occurred: " +
	    	                        e.getMessage());
                e.printStackTrace();
                return;
            }
            
        }
        
    }

    public void itemStateChanged(ItemEvent itemEvent) {

        int selectedIndex;
        String serverInformation = "";
        
        if(itemEvent.getStateChange() == ItemEvent.SELECTED) {

            selectedIndex = sessionList.getSelectedIndex();
        
            try {
                serverInformation = mainServer.getServerInformation(sessionList.getItem(selectedIndex));
            }
            catch(NotRunningException e) {
                System.out.println("MainApplet.itemStateChanged: an exception occurred: " +
		                            e.getMessage());
        	    e.printStackTrace();

        	    System.out.println("Remove this item from the sessionList.");
        	    sessionList.remove(selectedIndex);
            }
            
            catch(Exception e)
            {
                System.out.println("MainApplet.itemStateChanged: an exception occurred: " +
		                            e.getMessage());
        	    e.printStackTrace();
            }            
        
            sessionInfo.setText(serverInformation);
            return;
            
        }
        
        if(itemEvent.getStateChange() == ItemEvent.DESELECTED) {

            sessionInfo.setText("");
            return;
            
        }
    
    }
    
    protected void logout() {
        
        Object enum[];
        
        try {
            mainServer.logout(this);
        }
        catch(Exception e)
        {
            System.out.println("MainApplet.logout: an exception occurred: " +
		                        e.getMessage());
    	    e.printStackTrace();
        }
        
        clientsCritical.enter();

        enum = new Object[clients.size()];
        clients.copyInto(enum);
        
        try {
            for(int i = 0; i < enum.length; i ++)
                destroyClient((ApplicationClientInterface)enum[i], true);
        }
        catch(Exception e)
        {
            System.out.println("MainApplet.logout: an exception occurred: " +
		                        e.getMessage());
    	    e.printStackTrace();
        }
        
        clientsCritical.leave();

    }
    
    protected void updateLists() {

        Vector users, sessions;
                
        try {
            users = mainServer.getObserverAccounts();
            sessions = mainServer.getRunningServers();
        }
        catch(Exception e) {
            System.out.println("MainApplet.updateLists: an exception occurred: " +
		                        e.getMessage());
    	    e.printStackTrace();

    	    return;
        }

        userList.removeAll();
        for(int i = 0; i < users.size(); i ++)
            userList.add((String)users.elementAt(i));
            
        sessionList.removeAll();
        for(int i = 0; i < sessions.size(); i ++)
            sessionList.add((String)sessions.elementAt(i));

        sessionInfo.setText("");


    }
    
    /*
     * updateComponents - this method is used to enable or disable the components of the applet
     */
    protected void updateComponents() {
        
        userLabel.setEnabled(loggedIn);
        sessionLabel.setEnabled(loggedIn);
        infoLabel.setEnabled(loggedIn);
        userList.setEnabled(loggedIn);
        sessionList.setEnabled(loggedIn);
        sessionInfo.setEnabled(loggedIn);
        
        loginButton.setEnabled(!loggedIn);
        logoutButton.setEnabled(loggedIn);
        newSessionButton.setEnabled(loggedIn);

    }

}
