/**
 * file:    MainServerInterface.java
 *
 * classes: interface MainServerInterface
 *
 * purpose: - implementations of this interface can provide a server with a simple login/logout functionality
 *
 * date:    05/09/97  last change: 05/03/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */


package observer.main;

import java.rmi.*;
import java.util.*;

import observer.generic.*;

/**
 * interface ViewSubjectInterface
 *
 */
public interface MainServerInterface extends ViewSubjectInterface {

    /*
     * getRunningServers() returns the names of all running servers
     */
    Vector getRunningServers() throws RemoteException;
    
    /*
     * getServerInformation(String server) returns information about thge specified server
     */
    String getServerInformation(String serverName)
        throws RemoteException, NotRunningException;
    
    /*
     * getInstalledSystems() returns the names of systems currently installed on the server
     */
    Vector getInstalledSystems() throws RemoteException;
    
    /*
     * startServer() tries to start a server of the specified system using an array
     * of accounts to determine the observers that will participate
     */
    void startServer(String system, String sessionName, Vector clientAccounts)
        throws RemoteException, NotLoggedInException, NotInstalledException, AlreadyRunningException, UserMismatchException;
        
    void destroyServer(String sessionName) throws RemoteException, NotRunningException;
    
}
