/**
 * file:    NewSessionFrame.java
 *
 * classes: class NewSessionFrame
 *
 * purpose: - a frame which is used to start an application server
 *
 * date:    05/16/97  last change: 05/16/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */

package observer.main;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

public class NewSessionFrame extends CenteredFrame implements ActionListener, TextListener {
    
    protected boolean valid = false;
    
    protected FramedPanel mainPanel;
    protected Label sessionLabel;
    protected TextField sessionTextField;
    protected Label systemsLabel;
    protected Choice systemsChoice;
    protected FramedPanel buttonPanel;
    protected Button okButton, cancelButton;
    protected String sessionName = null;
    protected ActionListener actionListener;
    
    public NewSessionFrame(ActionListener l, Vector systems) {
        
        super("Create a new session...");
        
        actionListener = l;

        mainPanel = new FramedPanel(FramedPanel.NOFRAME);
        sessionLabel = new Label("Session name");
        sessionTextField = new TextField(40);
        systemsLabel = new Label("Available systems");
        systemsChoice = new Choice();
        buttonPanel = new FramedPanel(FramedPanel.FRAMEIN);
        okButton = new Button("Ok");
        cancelButton = new Button("Cancel");
        
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Insets insets = new Insets(0, 0, 0, 0);

        mainPanel.setLayout(gbl);

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 0;
        gbl.setConstraints(sessionLabel, gbc);    
        
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weighty = 1;
        gbl.setConstraints(sessionTextField, gbc);
        
        gbc.weighty = 0;
        insets.top = 10;
        gbc.insets = insets;
        gbl.setConstraints(systemsLabel, gbc);
        insets.top = 0;
        gbl.setConstraints(systemsChoice, gbc);
        
        buttonPanel.setLayout(new GridLayout(1, 0));
        buttonPanel.add(okButton);
        buttonPanel.add(cancelButton);
        
        gbc.gridx = 0;
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.gridwidth = 0;
        gbc.weightx = 1;
        gbc.weighty = 0;
        insets.top = 10;
        gbl.setConstraints(buttonPanel, gbc);
        
        mainPanel.add(sessionLabel);
        mainPanel.add(sessionTextField);
        mainPanel.add(systemsLabel);
        mainPanel.add(systemsChoice);
        mainPanel.add(buttonPanel);
            
        setLayout(new GridLayout(0, 1));
        add(mainPanel);

        addWindowListener(this);
        sessionTextField.addKeyListener(this);
        sessionTextField.addTextListener(this);
        systemsChoice.addKeyListener(this);
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);

        // now add available systems to the appropriate list
        for(int i = 0; i < systems.size(); i ++)
            systemsChoice.add((String)systems.elementAt(i));
        
        pack();
        
        updateComponents();
    
    }
    
    public void enterFrame() {

        if(valid) {
            sessionName = sessionTextField.getText();
        
            actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, ""));
        
            dispose();
        }

    }
    
    public void escapeFrame() {

        actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, ""));        
        
        dispose();
        
    }
        
    public void actionPerformed(ActionEvent e) {

        Object source = e.getSource();       

        if(source == okButton)
        {
            enterFrame();
            return;
        }
        
        if(source == cancelButton) {
            escapeFrame();
            return;
        }

    }
    
    public void textValueChanged(TextEvent e) {
        
        Object source = e.getSource();
        
        if(source == sessionTextField) {
            valid = (sessionTextField.getText().length() != 0);
            updateComponents();
            return;
        }
    }
    
    public String getSessionName() {
    
        return sessionName;
    
    }
    
    public String getSystemName() {
    
        return systemsChoice.getSelectedItem();
    
    }
    
    public void updateComponents() {

        okButton.setEnabled(valid);
        
    }
    
}
