/**
 * file:    SimpleQuestionFrame.java
 *
 * classes: class SimpleQuestionFrame
 *
 * purpose: - a frame that prompts a question to the user
 *
 * date:    05/16/97  last change: 05/16/97 18:00
 *
 * author:  Normen Seemann
 *
 * version: 1.00
 *
 */

package observer.main;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class SimpleQuestionFrame extends CenteredFrame implements ActionListener {
    
    protected FramedPanel mainPanel;
    protected Label messageLabel;
    protected List list;
    protected FramedPanel buttonPanel;
    protected Vector buttons;
    protected ActionListener actionListener;
    protected int selectedButton;
    
    public SimpleQuestionFrame(ActionListener l, String title, String mL, boolean sL, Vector buttonLabels) {
        
        super(title);
       
        Button button;
       
        actionListener = l;
  
        selectedButton = buttonLabels.size();
        
        mainPanel = new FramedPanel(FramedPanel.NOFRAME);
        messageLabel = new Label(mL);
        list = new List();

        buttonPanel = new FramedPanel(FramedPanel.FRAMEIN);
        buttonPanel.setLayout(new GridLayout(1, 0));
        buttons = new Vector(buttonLabels.size());
        for(int i = 0; i < buttonLabels.size(); i ++)
        {
            button = new Button((String)buttonLabels.elementAt(i));
            buttonPanel.add(button);
            button.addActionListener(this);
            buttons.addElement(button);
        }
        
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Insets insets = new Insets(0, 0, 0, 0);

        mainPanel.setLayout(gbl);

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbl.setConstraints(messageLabel, gbc);            

        gbc.gridx = 0;
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbl.setConstraints(list, gbc);
                        
        gbc.weightx = 1;
        gbc.weighty = 0;
        insets.top = 10;
        gbc.insets = insets;
        gbl.setConstraints(buttonPanel, gbc);
        
        mainPanel.add(messageLabel);
        if(sL)
            mainPanel.add(list);
        mainPanel.add(buttonPanel);
    
        addWindowListener(this);
        
        setLayout(new GridLayout(0, 1));
        add(mainPanel);

        pack();
    }
    
    public int getSelectedButton() {

        return selectedButton;
        
    }
    
    public void windowClosing(WindowEvent e) {
        
        actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, ""));        
        
        dispose();
        
    }

    public void windowOpened(WindowEvent e) {
    }
    
    public void actionPerformed(ActionEvent e) {

        Object source = e.getSource();
        
        for(int i = 0; i < buttons.size(); i ++)
            if(source == buttons.elementAt(i))
                selectedButton = i;
                
        actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, ""));
        
        dispose();

    }
    
    public void appendItems(Vector items) {
        
        for(int i = 0; i < items.size(); i ++)
            list.addItem((String)items.elementAt(i));
    
    }

}

